@@ -1,7 +1,5 @@
 # coding: utf-8
 
-from __future__ import absolute_import, unicode_literals, print_function
-
 import sys
 import os
 import warnings
@@ -18,7 +16,7 @@ from ruamel.yaml.nodes import *  # NOQA
 
 from ruamel.yaml.loader import BaseLoader, SafeLoader, Loader, RoundTripLoader  # NOQA
 from ruamel.yaml.dumper import BaseDumper, SafeDumper, Dumper, RoundTripDumper  # NOQA
-from ruamel.yaml.compat import StringIO, BytesIO, with_metaclass, PY3
+from ruamel.yaml.compat import StringIO, BytesIO, with_metaclass, nprint
 from ruamel.yaml.resolver import VersionedResolver, Resolver  # NOQA
 from ruamel.yaml.representer import (
     BaseRepresenter,
@@ -35,13 +33,9 @@ from ruamel.yaml.constructor import (
 from ruamel.yaml.loader import Loader as UnsafeLoader
 
 if False:  # MYPY
-    from typing import List, Set, Dict, Union, Any, Callable  # NOQA
+    from typing import List, Set, Dict, Union, Any, Callable, Optional, Text  # NOQA
     from ruamel.yaml.compat import StreamType, StreamTextType, VersionType  # NOQA
-
-    if PY3:
-        from pathlib import Path
-    else:
-        Path = Any
+    from pathlib import Path
 
 try:
     from _ruamel_yaml import CParser, CEmitter  # type: ignore
@@ -50,20 +44,15 @@ except:  # NOQA
 
 # import io
 
-enforce = object()
-
 
 # YAML is an acronym, i.e. spoken: rhymes with "camel". And thus a
 # subset of abbreviations, which should be all caps according to PEP8
 
 
 class YAML(object):
-    def __init__(
-        self, _kw=enforce, typ=None, pure=False, output=None, plug_ins=None  # input=None,
-    ):
+    def __init__(self, *, typ=None, pure=False, output=None, plug_ins=None):  # input=None,
         # type: (Any, Optional[Text], Any, Any, Any) -> None
         """
-        _kw: not used, forces keyword arguments in 2.7 (in 3 you can do (*, safe_load=..)
         typ: 'rt'/None -> RoundTripLoader/RoundTripDumper,  (default)
              'safe'    -> SafeLoader/SafeDumper,
              'unsafe'  -> normal/unsafe Loader/Dumper
@@ -72,13 +61,8 @@ class YAML(object):
         input/output: needed to work as context manager
         plug_ins: a list of plug-in files
         """
-        if _kw is not enforce:
-            raise TypeError(
-                '{}.__init__() takes no positional argument but at least '
-                'one was given ({!r})'.format(self.__class__.__name__, _kw)
-            )
 
-        self.typ = 'rt' if typ is None else typ
+        self.typ = ['rt'] if typ is None else (typ if isinstance(typ, list) else [typ])
         self.pure = pure
 
         # self._input = input
@@ -92,21 +76,16 @@ class YAML(object):
         self.Resolver = ruamel.yaml.resolver.VersionedResolver  # type: Any
         self.allow_unicode = True
         self.Reader = None  # type: Any
+        self.Representer = None  # type: Any
+        self.Constructor = None  # type: Any
         self.Scanner = None  # type: Any
         self.Serializer = None  # type: Any
         self.default_flow_style = None  # type: Any
-        if self.typ == 'rt':
-            self.default_flow_style = False
-            # no optimized rt-dumper yet
-            self.Emitter = ruamel.yaml.emitter.Emitter  # type: Any
-            self.Serializer = ruamel.yaml.serializer.Serializer  # type: Any
-            self.Representer = ruamel.yaml.representer.RoundTripRepresenter  # type: Any
-            self.Scanner = ruamel.yaml.scanner.RoundTripScanner  # type: Any
-            # no optimized rt-parser yet
-            self.Parser = ruamel.yaml.parser.RoundTripParser  # type: Any
-            self.Composer = ruamel.yaml.composer.Composer  # type: Any
-            self.Constructor = ruamel.yaml.constructor.RoundTripConstructor  # type: Any
-        elif self.typ == 'safe':
+        typ_found = 1
+        setup_rt = False
+        if 'rt' in self.typ:
+            setup_rt = True
+        elif 'safe' in self.typ:
             self.Emitter = (
                 ruamel.yaml.emitter.Emitter if pure or CEmitter is None else CEmitter
             )
@@ -114,13 +93,13 @@ class YAML(object):
             self.Parser = ruamel.yaml.parser.Parser if pure or CParser is None else CParser
             self.Composer = ruamel.yaml.composer.Composer
             self.Constructor = ruamel.yaml.constructor.SafeConstructor
-        elif self.typ == 'base':
+        elif 'base' in self.typ:
             self.Emitter = ruamel.yaml.emitter.Emitter
             self.Representer = ruamel.yaml.representer.BaseRepresenter
             self.Parser = ruamel.yaml.parser.Parser if pure or CParser is None else CParser
             self.Composer = ruamel.yaml.composer.Composer
             self.Constructor = ruamel.yaml.constructor.BaseConstructor
-        elif self.typ == 'unsafe':
+        elif 'unsafe' in self.typ:
             self.Emitter = (
                 ruamel.yaml.emitter.Emitter if pure or CEmitter is None else CEmitter
             )
@@ -129,14 +108,20 @@ class YAML(object):
             self.Composer = ruamel.yaml.composer.Composer
             self.Constructor = ruamel.yaml.constructor.Constructor
         else:
-            for module in self.plug_ins:
-                if getattr(module, 'typ', None) == self.typ:
-                    module.init_typ(self)
-                    break
-            else:
-                raise NotImplementedError(
-                    'typ "{}"not recognised (need to install plug-in?)'.format(self.typ)
-                )
+            setup_rt = True
+            typ_found = 0
+        if setup_rt:
+            self.default_flow_style = False
+            # no optimized rt-dumper yet
+            self.Emitter = ruamel.yaml.emitter.Emitter
+            self.Serializer = ruamel.yaml.serializer.Serializer
+            self.Representer = ruamel.yaml.representer.RoundTripRepresenter
+            self.Scanner = ruamel.yaml.scanner.RoundTripScanner
+            # no optimized rt-parser yet
+            self.Parser = ruamel.yaml.parser.RoundTripParser
+            self.Composer = ruamel.yaml.composer.Composer
+            self.Constructor = ruamel.yaml.constructor.RoundTripConstructor
+        del setup_rt
         self.stream = None
         self.canonical = None
         self.old_indent = None
@@ -146,6 +131,9 @@ class YAML(object):
         self.map_indent = None
         self.sequence_indent = None
         self.sequence_dash_offset = 0
+        self.compact_seq_seq = None
+        self.compact_seq_map = None
+        self.sort_base_mapping_type_on_output = None  # default: sort
 
         self.top_level_colon_align = None
         self.prefix_colon = None
@@ -158,22 +146,38 @@ class YAML(object):
         self.tags = None
         self.default_style = None
         self.top_level_block_style_scalar_no_indent_error_1_1 = False
+        # directives end indicator with single scalar document
+        self.scalar_after_indicator = None
+        # [a, b: 1, c: {d: 2}]  vs. [a, {b: 1}, {c: {d: 2}}]
+        self.brace_single_entry_mapping_in_flow_sequence = False
+        self.comment_handling = None
+        for module in self.plug_ins:
+            if getattr(module, 'typ', None) in self.typ:
+                typ_found += 1
+                module.init_typ(self)
+                break
+        if typ_found == 0:
+            raise NotImplementedError(
+                'typ "{}"not recognised (need to install plug-in?)'.format(self.typ)
+            )
 
     @property
     def reader(self):
         # type: () -> Any
-        attr = '_' + sys._getframe().f_code.co_name
-        if not hasattr(self, attr):
-            setattr(self, attr, self.Reader(None, loader=self))
-        return getattr(self, attr)
+        try:
+            return self._reader  # type: ignore
+        except AttributeError:
+            self._reader = self.Reader(None, loader=self)
+            return self._reader
 
     @property
     def scanner(self):
         # type: () -> Any
-        attr = '_' + sys._getframe().f_code.co_name
-        if not hasattr(self, attr):
-            setattr(self, attr, self.Scanner(loader=self))
-        return getattr(self, attr)
+        try:
+            return self._scanner  # type: ignore
+        except AttributeError:
+            self._scanner = self.Scanner(loader=self)
+            return self._scanner
 
     @property
     def parser(self):
@@ -237,6 +241,7 @@ class YAML(object):
                     allow_unicode=self.allow_unicode,
                     line_break=self.line_break,
                     prefix_colon=self.prefix_colon,
+                    brace_single_entry_mapping_in_flow_sequence=self.brace_single_entry_mapping_in_flow_sequence,  # NOQA
                     dumper=self,
                 )
                 setattr(self, attr, _emitter)
@@ -247,6 +252,10 @@ class YAML(object):
                 if self.sequence_dash_offset is not None:
                     _emitter.sequence_dash_offset = self.sequence_dash_offset
                     # _emitter.block_seq_indent = self.sequence_dash_offset
+                if self.compact_seq_seq is not None:
+                    _emitter.compact_seq_seq = self.compact_seq_seq
+                if self.compact_seq_map is not None:
+                    _emitter.compact_seq_map = self.compact_seq_map
             else:
                 if getattr(self, '_stream', None) is None:
                     # wait for the stream
@@ -278,17 +287,109 @@ class YAML(object):
         # type: () -> Any
         attr = '_' + sys._getframe().f_code.co_name
         if not hasattr(self, attr):
-            setattr(
-                self,
-                attr,
-                self.Representer(
-                    default_style=self.default_style,
-                    default_flow_style=self.default_flow_style,
-                    dumper=self,
-                ),
+            repres = self.Representer(
+                default_style=self.default_style,
+                default_flow_style=self.default_flow_style,
+                dumper=self,
             )
+            if self.sort_base_mapping_type_on_output is not None:
+                repres.sort_base_mapping_type_on_output = self.sort_base_mapping_type_on_output
+            setattr(self, attr, repres)
         return getattr(self, attr)
 
+    def scan(self, stream):
+        # type: (StreamTextType) -> Any
+        """
+        Scan a YAML stream and produce scanning tokens.
+        """
+        if not hasattr(stream, 'read') and hasattr(stream, 'open'):
+            # pathlib.Path() instance
+            with stream.open('rb') as fp:
+                return self.scan(fp)
+        _, parser = self.get_constructor_parser(stream)
+        try:
+            while self.scanner.check_token():
+                yield self.scanner.get_token()
+        finally:
+            parser.dispose()
+            try:
+                self._reader.reset_reader()
+            except AttributeError:
+                pass
+            try:
+                self._scanner.reset_scanner()
+            except AttributeError:
+                pass
+
+    def parse(self, stream):
+        # type: (StreamTextType) -> Any
+        """
+        Parse a YAML stream and produce parsing events.
+        """
+        if not hasattr(stream, 'read') and hasattr(stream, 'open'):
+            # pathlib.Path() instance
+            with stream.open('rb') as fp:
+                return self.parse(fp)
+        _, parser = self.get_constructor_parser(stream)
+        try:
+            while parser.check_event():
+                yield parser.get_event()
+        finally:
+            parser.dispose()
+            try:
+                self._reader.reset_reader()
+            except AttributeError:
+                pass
+            try:
+                self._scanner.reset_scanner()
+            except AttributeError:
+                pass
+
+    def compose(self, stream):
+        # type: (Union[Path, StreamTextType]) -> Any
+        """
+        Parse the first YAML document in a stream
+        and produce the corresponding representation tree.
+        """
+        if not hasattr(stream, 'read') and hasattr(stream, 'open'):
+            # pathlib.Path() instance
+            with stream.open('rb') as fp:
+                return self.load(fp)
+        constructor, parser = self.get_constructor_parser(stream)
+        try:
+            return constructor.composer.get_single_node()
+        finally:
+            parser.dispose()
+            try:
+                self._reader.reset_reader()
+            except AttributeError:
+                pass
+            try:
+                self._scanner.reset_scanner()
+            except AttributeError:
+                pass
+
+    def compose_all(self, stream):
+        # type: (Union[Path, StreamTextType]) -> Any
+        """
+        Parse all YAML documents in a stream
+        and produce corresponding representation trees.
+        """
+        constructor, parser = self.get_constructor_parser(stream)
+        try:
+            while constructor.composer.check_node():
+                yield constructor.composer.get_node()
+        finally:
+            parser.dispose()
+            try:
+                self._reader.reset_reader()
+            except AttributeError:
+                pass
+            try:
+                self._scanner.reset_scanner()
+            except AttributeError:
+                pass
+
     # separate output resolver?
 
     # def load(self, stream=None):
@@ -313,7 +414,7 @@ class YAML(object):
         """
         if not hasattr(stream, 'read') and hasattr(stream, 'open'):
             # pathlib.Path() instance
-            with stream.open('r') as fp:  # type: ignore
+            with stream.open('rb') as fp:
                 return self.load(fp)
         constructor, parser = self.get_constructor_parser(stream)
         try:
@@ -321,25 +422,20 @@ class YAML(object):
         finally:
             parser.dispose()
             try:
-                self._reader.reset_reader()  # type: ignore
+                self._reader.reset_reader()
             except AttributeError:
                 pass
             try:
-                self._scanner.reset_scanner()  # type: ignore
+                self._scanner.reset_scanner()
             except AttributeError:
                 pass
 
-    def load_all(self, stream, _kw=enforce):  # , skip=None):
+    def load_all(self, stream):  # *, skip=None):
         # type: (Union[Path, StreamTextType], Any) -> Any
-        if _kw is not enforce:
-            raise TypeError(
-                '{}.__init__() takes no positional argument but at least '
-                'one was given ({!r})'.format(self.__class__.__name__, _kw)
-            )
         if not hasattr(stream, 'read') and hasattr(stream, 'open'):
             # pathlib.Path() instance
-            with stream.open('r') as fp:  # type: ignore
-                for d in self.load_all(fp, _kw=enforce):
+            with stream.open('r') as fp:
+                for d in self.load_all(fp):
                     yield d
                 return
         # if skip is None:
@@ -353,11 +449,11 @@ class YAML(object):
         finally:
             parser.dispose()
             try:
-                self._reader.reset_reader()  # type: ignore
+                self._reader.reset_reader()
             except AttributeError:
                 pass
             try:
-                self._scanner.reset_scanner()  # type: ignore
+                self._scanner.reset_scanner()
             except AttributeError:
                 pass
 
@@ -389,33 +485,69 @@ class YAML(object):
                 # if you just initialise the CParser, to much of resolver.py
                 # is actually used
                 rslvr = self.Resolver
-                if rslvr is ruamel.yaml.resolver.VersionedResolver:
-                    rslvr = ruamel.yaml.resolver.Resolver
+                # if rslvr is ruamel.yaml.resolver.VersionedResolver:
+                #     rslvr = ruamel.yaml.resolver.Resolver
 
                 class XLoader(self.Parser, self.Constructor, rslvr):  # type: ignore
-                    def __init__(selfx, stream, version=None, preserve_quotes=None):
+                    def __init__(selfx, stream, version=self.version, preserve_quotes=None):
                         # type: (StreamTextType, Optional[VersionType], Optional[bool]) -> None  # NOQA
                         CParser.__init__(selfx, stream)
                         selfx._parser = selfx._composer = selfx
                         self.Constructor.__init__(selfx, loader=selfx)
                         selfx.allow_duplicate_keys = self.allow_duplicate_keys
-                        rslvr.__init__(selfx, loadumper=selfx)
+                        rslvr.__init__(selfx, version=version, loadumper=selfx)
 
                 self._stream = stream
                 loader = XLoader(stream)
                 return loader, loader
         return self.constructor, self.parser
 
-    def dump(self, data, stream=None, _kw=enforce, transform=None):
+    def emit(self, events, stream):
+        """
+        Emit YAML parsing events into a stream.
+        If stream is None, return the produced string instead.
+        """
+        _, _, emitter = self.get_serializer_representer_emitter(stream, None)
+        try:
+            for event in events:
+                emitter.emit(event)
+        finally:
+            try:
+                emitter.dispose()
+            except AttributeError:
+                raise
+
+    def serialize(self, node, stream):
+        # type: (Any, Optional[StreamType]) -> Any
+        """
+        Serialize a representation tree into a YAML stream.
+        If stream is None, return the produced string instead.
+        """
+        self.serialize_all([node], stream)
+
+    def serialize_all(self, nodes, stream):
+        # type: (Any, Optional[StreamType]) -> Any
+        """
+        Serialize a sequence of representation trees into a YAML stream.
+        If stream is None, return the produced string instead.
+        """
+        serializer, _, emitter = self.get_serializer_representer_emitter(stream, None)
+        try:
+            serializer.open()
+            for node in nodes:
+                serializer.serialize(node)
+            serializer.close()
+        finally:
+            try:
+                emitter.dispose()
+            except AttributeError:
+                raise
+
+    def dump(self, data, stream=None, *, transform=None):
         # type: (Any, Union[Path, StreamType], Any, Any) -> Any
         if self._context_manager:
             if not self._output:
                 raise TypeError('Missing output stream while dumping from context manager')
-            if _kw is not enforce:
-                raise TypeError(
-                    '{}.dump() takes one positional argument but at least '
-                    'two were given ({!r})'.format(self.__class__.__name__, _kw)
-                )
             if transform is not None:
                 raise TypeError(
                     '{}.dump() in the context manager cannot have transform keyword '
@@ -425,17 +557,12 @@ class YAML(object):
         else:  # old style
             if stream is None:
                 raise TypeError('Need a stream argument when not dumping from context manager')
-            return self.dump_all([data], stream, _kw, transform=transform)
+            return self.dump_all([data], stream, transform=transform)
 
-    def dump_all(self, documents, stream, _kw=enforce, transform=None):
+    def dump_all(self, documents, stream, *, transform=None):
         # type: (Any, Union[Path, StreamType], Any, Any) -> Any
         if self._context_manager:
             raise NotImplementedError
-        if _kw is not enforce:
-            raise TypeError(
-                '{}.dump(_all) takes two positional argument but at least '
-                'three were given ({!r})'.format(self.__class__.__name__, _kw)
-            )
         self._output = stream
         self._context_manager = YAMLContextManager(self, transform=transform)
         for data in documents:
@@ -444,20 +571,15 @@ class YAML(object):
         self._output = None
         self._context_manager = None
 
-    def Xdump_all(self, documents, stream, _kw=enforce, transform=None):
+    def Xdump_all(self, documents, stream, *, transform=None):
         # type: (Any, Union[Path, StreamType], Any, Any) -> Any
         """
         Serialize a sequence of Python objects into a YAML stream.
         """
         if not hasattr(stream, 'write') and hasattr(stream, 'open'):
             # pathlib.Path() instance
-            with stream.open('w') as fp:  # type: ignore
-                return self.dump_all(documents, fp, _kw, transform=transform)
-        if _kw is not enforce:
-            raise TypeError(
-                '{}.dump(_all) takes two positional argument but at least '
-                'three were given ({!r})'.format(self.__class__.__name__, _kw)
-            )
+            with stream.open('w') as fp:
+                return self.dump_all(documents, fp, transform=transform)
         # The stream should have the methods `write` and possibly `flush`.
         if self.top_level_colon_align is True:
             tlca = max([len(str(x)) for x in documents[0]])  # type: Any
@@ -490,7 +612,7 @@ class YAML(object):
             delattr(self, '_serializer')
             delattr(self, '_emitter')
         if transform:
-            val = stream.getvalue()  # type: ignore
+            val = stream.getvalue()
             if self.encoding:
                 val = val.decode(self.encoding)
             if fstream is None:
@@ -507,18 +629,22 @@ class YAML(object):
                 self.Serializer = ruamel.yaml.serializer.Serializer
             self.emitter.stream = stream
             self.emitter.top_level_colon_align = tlca
+            if self.scalar_after_indicator is not None:
+                self.emitter.scalar_after_indicator = self.scalar_after_indicator
             return self.serializer, self.representer, self.emitter
         if self.Serializer is not None:
             # cannot set serializer with CEmitter
             self.Emitter = ruamel.yaml.emitter.Emitter
             self.emitter.stream = stream
             self.emitter.top_level_colon_align = tlca
+            if self.scalar_after_indicator is not None:
+                self.emitter.scalar_after_indicator = self.scalar_after_indicator
             return self.serializer, self.representer, self.emitter
         # C routines
 
         rslvr = (
             ruamel.yaml.resolver.BaseResolver
-            if self.typ == 'base'
+            if 'base' in self.typ
             else ruamel.yaml.resolver.Resolver
         )
 
@@ -584,7 +710,7 @@ class YAML(object):
     # basic types
     def map(self, **kw):
         # type: (Any) -> Any
-        if self.typ == 'rt':
+        if 'rt' in self.typ:
             from ruamel.yaml.comments import CommentedMap
 
             return CommentedMap(**kw)
@@ -593,7 +719,7 @@ class YAML(object):
 
     def seq(self, *args):
         # type: (Any) -> Any
-        if self.typ == 'rt':
+        if 'rt' in self.typ:
             from ruamel.yaml.comments import CommentedSeq
 
             return CommentedSeq(*args)
@@ -603,7 +729,16 @@ class YAML(object):
     # helpers
     def official_plug_ins(self):
         # type: () -> Any
-        bd = os.path.dirname(__file__)
+        """search for list of subdirs that are plug-ins, if __file__ is not available, e.g.
+        single file installers that are not properly emulating a file-system (issue 324)
+        no plug-ins will be found. If any are packaged, you know which file that are
+        and you can explicitly provide it during instantiation:
+            yaml = ruamel.yaml.YAML(plug_ins=['ruamel/yaml/jinja2/__plug_in__'])
+        """
+        try:
+            bd = os.path.dirname(__file__)
+        except NameError:
+            return []
         gpbd = os.path.dirname(os.path.dirname(bd))
         res = [x.replace(gpbd, "")[1:-3] for x in glob.glob(bd + '/*/__plug_in__.py')]
         return res
@@ -639,26 +774,6 @@ class YAML(object):
             self.constructor.add_constructor(tag, f_y)
         return cls
 
-    def parse(self, stream):
-        # type: (StreamTextType) -> Any
-        """
-        Parse a YAML stream and produce parsing events.
-        """
-        _, parser = self.get_constructor_parser(stream)
-        try:
-            while parser.check_event():
-                yield parser.get_event()
-        finally:
-            parser.dispose()
-            try:
-                self._reader.reset_reader()  # type: ignore
-            except AttributeError:
-                pass
-            try:
-                self._scanner.reset_scanner()  # type: ignore
-            except AttributeError:
-                pass
-
     # ### context manager
 
     def __enter__(self):
@@ -704,10 +819,15 @@ class YAML(object):
         # type: (Any) -> None
         self.sequence_dash_offset = val
 
+    def compact(self, seq_seq=None, seq_map=None):
+        # type: (Any, Any) -> None
+        self.compact_seq_seq = seq_seq
+        self.compact_seq_map = seq_map
+
 
 class YAMLContextManager(object):
     def __init__(self, yaml, transform=None):
-        # type: (Any, Optional[Callable]) -> None
+        # type: (Any, Any) -> None  # used to be: (Any, Optional[Callable]) -> None
         self._yaml = yaml
         self._output_inited = False
         self._output_path = None
@@ -850,6 +970,20 @@ def yaml_object(yml):
 
 
 ########################################################################################
+def warn_deprecation(fun, method, arg=''):
+    from ruamel.yaml.compat import _F
+
+    warnings.warn(
+        _F(
+            '\n{fun} will be removed, use\n\n  yaml=YAML({arg})\n  yaml.{method}(...)\n\ninstead',  # NOQA
+            fun=fun,
+            method=method,
+            arg=arg,
+        ),
+        PendingDeprecationWarning,  # this will show when testing with pytest/tox
+        stacklevel=3,
+    )
+########################################################################################
 
 
 def scan(stream, Loader=Loader):
@@ -857,6 +991,7 @@ def scan(stream, Loader=Loader):
     """
     Scan a YAML stream and produce scanning tokens.
     """
+    warn_deprecation('scan', 'scan', arg="typ='unsafe', pure=True")
     loader = Loader(stream)
     try:
         while loader.scanner.check_token():
@@ -870,6 +1005,7 @@ def parse(stream, Loader=Loader):
     """
     Parse a YAML stream and produce parsing events.
     """
+    warn_deprecation('parse', 'parse', arg="typ='unsafe', pure=True")
     loader = Loader(stream)
     try:
         while loader._parser.check_event():
@@ -884,6 +1020,7 @@ def compose(stream, Loader=Loader):
     Parse the first YAML document in a stream
     and produce the corresponding representation tree.
     """
+    warn_deprecation('compose', 'compose', arg="typ='unsafe', pure=True")
     loader = Loader(stream)
     try:
         return loader.get_single_node()
@@ -897,6 +1034,7 @@ def compose_all(stream, Loader=Loader):
     Parse all YAML documents in a stream
     and produce corresponding representation trees.
     """
+    warn_deprecation('compose', 'compose', arg="typ='unsafe', pure=True")
     loader = Loader(stream)
     try:
         while loader.check_node():
@@ -911,6 +1049,7 @@ def load(stream, Loader=None, version=None, preserve_quotes=None):
     Parse the first YAML document in a stream
     and produce the corresponding Python object.
     """
+    warn_deprecation('load', 'load', arg="typ='unsafe', pure=True")
     if Loader is None:
         warnings.warn(UnsafeLoaderWarning.text, UnsafeLoaderWarning, stacklevel=2)
         Loader = UnsafeLoader
@@ -935,6 +1074,7 @@ def load_all(stream, Loader=None, version=None, preserve_quotes=None):
     Parse all YAML documents in a stream
     and produce corresponding Python objects.
     """
+    warn_deprecation('load_all', 'load_all', arg="typ='unsafe', pure=True")
     if Loader is None:
         warnings.warn(UnsafeLoaderWarning.text, UnsafeLoaderWarning, stacklevel=2)
         Loader = UnsafeLoader
@@ -961,6 +1101,7 @@ def safe_load(stream, version=None):
     and produce the corresponding Python object.
     Resolve only basic YAML tags.
     """
+    warn_deprecation('safe_load', 'load', arg="typ='safe', pure=True")
     return load(stream, SafeLoader, version)
 
 
@@ -971,6 +1112,7 @@ def safe_load_all(stream, version=None):
     and produce corresponding Python objects.
     Resolve only basic YAML tags.
     """
+    warn_deprecation('safe_load_all', 'load_all', arg="typ='safe', pure=True")
     return load_all(stream, SafeLoader, version)
 
 
@@ -981,6 +1123,7 @@ def round_trip_load(stream, version=None, preserve_quotes=None):
     and produce the corresponding Python object.
     Resolve only basic YAML tags.
     """
+    warn_deprecation('round_trip_load_all', 'load')
     return load(stream, RoundTripLoader, version, preserve_quotes=preserve_quotes)
 
 
@@ -991,6 +1134,7 @@ def round_trip_load_all(stream, version=None, preserve_quotes=None):
     and produce corresponding Python objects.
     Resolve only basic YAML tags.
     """
+    warn_deprecation('round_trip_load_all', 'load_all')
     return load_all(stream, RoundTripLoader, version, preserve_quotes=preserve_quotes)
 
 
@@ -1009,6 +1153,7 @@ def emit(
     Emit YAML parsing events into a stream.
     If stream is None, return the produced string instead.
     """
+    warn_deprecation('emit', 'emit', arg="typ='safe', pure=True")
     getvalue = None
     if stream is None:
         stream = StringIO()
@@ -1034,7 +1179,7 @@ def emit(
         return getvalue()
 
 
-enc = None if PY3 else 'utf-8'
+enc = None
 
 
 def serialize_all(
@@ -1057,6 +1202,7 @@ def serialize_all(
     Serialize a sequence of representation trees into a YAML stream.
     If stream is None, return the produced string instead.
     """
+    warn_deprecation('serialize_all', 'serialize_all', arg="typ='safe', pure=True")
     getvalue = None
     if stream is None:
         if encoding is None:
@@ -1098,6 +1244,7 @@ def serialize(node, stream=None, Dumper=Dumper, **kwds):
     Serialize a representation tree into a YAML stream.
     If stream is None, return the produced string instead.
     """
+    warn_deprecation('serialize', 'serialize', arg="typ='safe', pure=True")
     return serialize_all([node], stream, Dumper=Dumper, **kwds)
 
 
@@ -1126,6 +1273,7 @@ def dump_all(
     Serialize a sequence of Python objects into a YAML stream.
     If stream is None, return the produced string instead.
     """
+    warn_deprecation('dump_all', 'dump_all', arg="typ='unsafe', pure=True")
     getvalue = None
     if top_level_colon_align is True:
         top_level_colon_align = max([len(str(x)) for x in documents[0]])
@@ -1199,6 +1347,7 @@ def dump(
     default_style ∈ None, '', '"', "'", '|', '>'
 
     """
+    warn_deprecation('dump', 'dump', arg="typ='unsafe', pure=True")
     return dump_all(
         [data],
         stream,
@@ -1226,6 +1375,7 @@ def safe_dump_all(documents, stream=None, **kwds):
     Produce only basic YAML tags.
     If stream is None, return the produced string instead.
     """
+    warn_deprecation('safe_dump_all', 'dump_all', arg="typ='safe', pure=True")
     return dump_all(documents, stream, Dumper=SafeDumper, **kwds)
 
 
@@ -1236,6 +1386,7 @@ def safe_dump(data, stream=None, **kwds):
     Produce only basic YAML tags.
     If stream is None, return the produced string instead.
     """
+    warn_deprecation('safe_dump', 'dump', arg="typ='safe', pure=True")
     return dump_all([data], stream, Dumper=SafeDumper, **kwds)
 
 
@@ -1261,6 +1412,7 @@ def round_trip_dump(
 ):
     # type: (Any, Optional[StreamType], Any, Any, Any, Optional[bool], Optional[int], Optional[int], Optional[bool], Any, Any, Optional[bool], Optional[bool], Optional[VersionType], Any, Any, Any, Any) -> Optional[str]   # NOQA
     allow_unicode = True if allow_unicode is None else allow_unicode
+    warn_deprecation('round_trip_dump', 'dump')
     return dump_all(
         [data],
         stream,
@@ -1463,7 +1615,7 @@ class YAMLObjectMetaclass(type):
 
     def __init__(cls, name, bases, kwds):
         # type: (Any, Any, Any) -> None
-        super(YAMLObjectMetaclass, cls).__init__(name, bases, kwds)
+        super().__init__(name, bases, kwds)
         if 'yaml_tag' in kwds and kwds['yaml_tag'] is not None:
             cls.yaml_constructor.add_constructor(cls.yaml_tag, cls.from_yaml)  # type: ignore
             cls.yaml_representer.add_representer(cls, cls.to_yaml)  # type: ignore
